#pragma once

//! \file SSRegisterPlugin.hpp

#ifndef ShutterStream_RegisterPlugin
#define ShutterStream_RegisterPlugin

#include "ShutterSreamGlobals.h"

#ifndef SS_NATIVE_API
#include <QtQml/QQmlEngine>
#include <QtQml/QQmlExtensionPlugin>
#endif

/*! \class SSRegisterPlugin
 \brief base ShutterStream plug-ins class

 A more detailed class description.
 */

class SSAbstractPlugin;
class SHUTTERSTREAM_API SSRegisterPlugin
#ifndef SS_NATIVE_API
    : public QQmlExtensionPlugin
#endif
{
public:
    //! plug-in type
    enum class PluginType : uint16_t
    {
        CameraHardware = 0x0001, //!< camera control plug-in
        TurntableHardware = 0x0002, //!< turntable control plug-in
        ToolsAction = 0x0004, //!< tool action control plug-in
        ImageProcessing = 0x0008, //!< editing image plug-in
        ImageViewEvents = 0x0010 //!< image view events plug-in
    };

    //! binary info
    struct BinaryInfo
    {
        uint32_t compilerVersion;
        uint32_t pluginVersion;
        bool isReleaseBuild;
    };

    SSRegisterPlugin();
    virtual ~SSRegisterPlugin();

    //! every plug-in is queried about supported type
    /*!
     \return OR combination of PluginType values
     */
    virtual uint32_t pluginAvailableTypes() const = 0;

    //! the plug-in is asked to create an object of designated type
    /*!
     \return an created plugin object
     \note ShutterStream take ownership of the created plugin
     */
    virtual SSAbstractPlugin* createPlugin(PluginType type) = 0;

    //! plug-in binary info
    static void pluginBinaryInfo(BinaryInfo* info);


#ifndef SS_NATIVE_API
    Q_OBJECT
protected:
    //! empty helper function not forcing plug-ins to register Qt Qml types
    /*
     \param uri plugin module name (first line from qmldir file)
     */
    virtual void registerTypes(const char* uri) override;
#endif    
};


#if defined(SS_NATIVE_API)
#ifdef SS_REGISTER_PLUGIN_EXPORT
/*! \fn ShutterStreamPlugin* registerShutterStreamPlugin()
 \brief developers of ShutterStream plug-ins must implement this function.
 \return pointer to SSRegisterPlugin class, ShutterStream take ownership of registered plugin
 */
SS_REGISTER_PLUGIN_EXPORT SSRegisterPlugin* registerShutterStreamPlugin();
#endif
#else
//! global app qml engine
SHUTTERSTREAM_API QQmlEngine* globalQmlEngine();
//! plug-ins interface
#define SSRegisterPlugin_iid "com.iconasys.SSRegisterPlugin"
Q_DECLARE_INTERFACE(SSRegisterPlugin, SSRegisterPlugin_iid)
#endif

#endif // ShutterStream_RegisterPlugin
