#pragma once

#ifndef ShutterStream_AbstractPlugin
#define ShutterStream_AbstractPlugin

#define SS_THREAD_SAFEIMPL
#define SS_SIGNAL SS_THREAD_SAFEIMPL
#define SS_SLOT SS_SIGNAL

#define DECLARE_SSPIMPL(CLASS_NAME) \
	public: \
		struct SS##CLASS_NAME##Private; \
	private: \
		SS##CLASS_NAME##Private* const m_PImpl;

#include "SSRegisterPlugin.h"

#include <stdint.h>
#include <stdlib.h>

#include <string>
#include <list>
#include <map>

class SHUTTERSTREAM_API SSAbstractPlugin
{
public:
	typedef std::list<SSAbstractPlugin*> PluginsList;
	typedef PluginsList::const_iterator PluginIterator;

	typedef std::list<std::string> DevicesNamesList;
	typedef DevicesNamesList::const_iterator DeviceNameIterator;

	typedef std::list<SSString> StringsList;

	typedef StringsList FilesNamesList;
	typedef FilesNamesList::const_iterator FileNameIterator;
	typedef SSString DirectoryPath;
	typedef std::map<DirectoryPath, FilesNamesList> DirectoryFilesNamesMap;

	virtual ~SSAbstractPlugin();

	//! called by host to know the plug-in type
	virtual SSRegisterPlugin::PluginType pluginType() const = 0;

	//! called by host to inform the plug-in about initialization event
	virtual void pluginDidInitialized();
	//! called by host to inform the plug-in about closing down event
	virtual void pluginWillClose();

	//! called by host to inform the plug-in when application become active
	virtual void applicationDidBecomeActive();
	//! called by host to inform the plug-in when application entered in background mode
	virtual void applicationDidEnterBackground();

	//! called by the host to inform plug-in when previously created UI frame view is about to be destroyed
	/*! here the plug-in can save the Ui settings
	\param frameView the UI frame view previously created
	\return true if host can close/destroy the view, false if not
	\sa applicationLocalDataLocation
	\sa setLastError
	 */
	virtual bool pluginUIFrameViewWillClose(SSUIFrameView frameView);

	//! called by the plug-in to set an error string (if any)
	void setLastError(SSString const errorString);

	//! called by host to retrieve last error string
	const SSString lastError() const;

	//! called by the plug-in to ask the host to build a list with desired plug-ins type
	static PluginsList availablePluginsList(SSRegisterPlugin::PluginType pluginType);

	//! helper strings character set conversion
	static std::wstring utf8ToWideString(const std::string& utf8String);
	static std::string wideToUtf8String(const std::wstring& wideString);
	static std::wstring nativeToWideString(SSString const nativeString);
	static SSString wideToNativeString(const std::wstring& wideString);

	//! application local data location
	static SSString applicationLocalDataLocation();

	//! current project directory
	static DirectoryPath currentProjectDirectory();

protected:
	SSAbstractPlugin();

	DECLARE_SSPIMPL(AbstractPlugin)
};

#endif
