#pragma once

//#ifndef SSIconasysTurntableControl_h
//#define SSIconasysTurntableControl_h

#include "SSTurntableHardware.h"

class SSFTDIController;

//// we have a stand-alone TurntableSDK for accessing the turntable hardware, so we need to make the class export-able
//#ifdef ICONASYS_TURNTABLE_EXPORTS
//class SHUTTERSTREAM_API SSIconasysTurntableControl
//#else
class SSIconasysTurntableControl
//#endif
	: public SSTurntableHardware
{
public:
	SSIconasysTurntableControl();
	~SSIconasysTurntableControl();

	virtual DevicesNamesList availableDevices() const override;

	virtual void openDevice(DevicesNamesList::size_type index) override; // do not use it in stand-alone sdk
	void openDevice(DevicesNamesList::size_type index, bool* success);
	virtual void closeDevice() override;

	virtual bool rotateDeviceToAngle(float anglePos) override;
	virtual void stopsSpinning() override;

	virtual SSUIFrameView createTurntableUIFrameView() override;
	virtual bool pluginUIFrameViewWillClose(SSUIFrameView frameView) override;
	virtual bool complete360ProductPhotographyWillStart(uint16_t* imagesCount, CustomAnglesArray* customAngles = nullptr) override;

	/// start & stop rotation
	bool startRotation();
	void stopRotation();

	/// A negative number will rotate opposite to what the direction is set,
	/// if the table supports it.  A rotation of 0 counts means continuous
	/// rotation.
	int SetRotationIncrementInCounts(int iRotateCounts);
	/// A negative number will rotate opposite to what the direction is set,
	/// if the table supports it.  Rotation may not be exact if table does
	/// not have appropriate resolution.  Roundoffs may occur and the return
	/// tells how many degrees the table actually set to rotate.  A rotation
	/// of zero degrees means continuous rotation.
	double SetRotationIncrementInDegrees(double dRotateDegrees);
	/// An additional helper function for setting continuous rotation
	void SetRotationContinuous() { SetRotationIncrementInCounts(0); }
	/// A global position should be used when making multiple rotations.
	/// Global position uses geometric angles with positive angles meaning
	/// counter-clockwise and negative meaning clockwise.
	void SetRotationNextGlobalPositionInDegrees(double dGlobalDegree);
	/// set global origin now
	void SetRotationGlobalOriginHere();
	/// Set number of counts per rotation.  Can be pulses, or encoder counts
	void SetCountsPerRevolution(unsigned int nCountsPerRotation);
	unsigned int countsPerRevolution() const;
	/// Let hardware know if a HIGH on the direction pin is clockwise.
	/// True if high is clockwise and false if high is counterclocwise
	void SetClockwiseActiveHigh(bool bHighIsClocwise = true);
	/// A helper function to more directly set a low for clockwise
	void SetClockwiseActiveLow(bool bLowIsClockwise = true)
	{
		SetClockwiseActiveHigh(!bLowIsClockwise);
	}
	bool isClockwiseActiveHigh() const;
	/// Let hardware know if a HIGH on the enable pin is enable.
	/// True if high is enable and false if high is disable
	void SetMotorEnableActiveHigh(bool bHighIsEnable = true);
	/// A helper function to more directly set a low for enable
	void SetMotorEnableActiveLow(bool bLowIsEnable = true)
	{
		SetMotorEnableActiveHigh(!bLowIsEnable);
	}
	bool isMotorEnableActiveHigh() const;
	/// An unsigned integer that translates into speed in the representative
	/// subclasses (usually pulse width).  Zero is fastest.
	/// pulse width is a 0 to 10 integer, with zero being fastest turn.
	void SetRotationSpeed(unsigned int nSpeed);
	unsigned int rotationSpeed() const;

	/// Set direction
	void SetRotationClockwise();
	/// Set direction
	void SetRotationCounterClockwise();
	/// Get direction
	bool isRotationClockwise() const;

	/// Set whether or not to lock the motor when idle
	void SetLockMotorWhenIdle(bool bLock);
	bool isLockMotorWhenIdle() const;
	/// Get if it's still rotating
	bool IsTurntableRotating();

	// turns count related
	void setComplete360TurnsCount(uint16_t complete360RotationTurnsCount);
	uint16_t complete360TurnsCount()const;

	// delay seconds
	float preDelaySeconds() const;
	float postDelaySeconds() const;
	void delaySeconds(float* preDelaySeconds, float* postDelaySeconds) const;
	void setDelaySeconds(float preDelaySeconds, float postDelaySeconds);

	// shutter release signal
	bool performShutterRelease();

	// turntable rotation is stopped (this is called from FTDI controller thread)
	void turntableRotationStoppedHelper();

	// save Ui settings for internal/private use
	void saveUISettingsInternalUse();

	// helper for checking if turntable is opened
	bool isTurntableOpened() const;

	// custom angles
	bool isUsingCustomAngles() const;
	void setUsingCustomAngles(bool useCustomAngles);

private:
	// common initialize function
	void initialize();
	// common event about turntable is about to start
	void complete360ProductPhotographyWillStart();
	// helper function for delayed rotate
	void rotateDeviceDelayed();
	// delay the rotate
	void delayRotatingDevice(float seconds);
	// sync UI with the opened device
	void syncOpenedDeviceUI(int index/* -1 means current turntable index is stopped rotating*/);

	//! FTDI controller
	SSFTDIController* m_ftdichipController;

DECLARE_SSPIMPL(IconasysTurntableControl)
};

//#endif /* SSIconasysTurntable_hpp */
